@@ -41,12 +41,6 @@ setMethod(f="getMovebank",
 	  definition = function(entity_type, login, ...){
 		  tmp <- list(...)
 		  url <- paste("://www.movebank.org/movebank/service/direct-read?entity_type=",entity_type  ,sep="")
-		  try(if(any(names(tmp)=="id")&class(unlist(tmp['id']))=="character") 
-		      tmp['id'] <- getMovebankID(unlist(tmp['id']), login) ,silent=T)
-		  try(if(any(names(tmp)=="study_id")&class(unlist(tmp['study_id']))=="character") 
-		      tmp['study_id'] <- getMovebankID(unlist(tmp['study_id']), login) ,silent=T)
-		  try(if(any(names(tmp)=="tag_study_id")&class(unlist(tmp['tag_study_id']))=="character") 
-		      tmp['tag_study_id'] <- getMovebankID(unlist(tmp['tag_study_id']), login) ,silent=T)
 		  if(length(tmp)!=0){
 			  tmp <- lapply(tmp, paste, collapse='%2C')
 			  url <- paste(url, sep="&",paste(names(tmp),tmp, collapse="&", sep="="))
@@ -64,8 +58,9 @@ setMethod(f="getMovebank",
 			  data <- read.csv(url, header=T, sep=",", as.is=T)
 		  }
 		  if(grepl(pattern="You.may.only.download.it.if.you.agree.with.the.terms", x=names(data)[1])) stop("You need a permission to access this data set. Go to www.movebank.org and accept the license terms when downloading the data set (you only have to do this once per data set).")
-		  if (grepl(pattern="X.html..head..title.Apache.Tomcat", capture.output(data)[1])==TRUE) stop("It looks like you are not allowed to download this data set.")
-		  if (grepl(pattern="are.not.available.for.download", capture.output(data)[1])==TRUE) stop("You have no permission to download this data set.")
+		  if (grepl(pattern="X.html..head..title.Apache.Tomcat", capture.output(data)[1])) stop("It looks like you are not allowed to download this data set. Or there is a sensor for which no attributes are available.")
+		  if (grepl(pattern="are.not.available.for.download", capture.output(data)[1])) stop("You have no permission to download this data set.")
+		  if (any(grepl(pattern="503 Service Temporarily Unavailable", unlist(head(data))))) stop("Movebank is (temporarily) unavailable")
 		  return(data)
 	  })
 
@@ -83,7 +78,7 @@ setMethod(f="searchMovebankStudies",
 	  signature=c(x="character",login="MovebankLogin"), 
 	  definition = function(x,login){  
 		  data <- getMovebank("study", login, sort="name", attributes="id%2Cname%2Ci_am_owner%2Ci_can_see_data%2Cthere_are_data_which_i_cannot_see")
-		  res <- c(data$name[grepl(x,data$name,useBytes=TRUE)])
+		  res <- as.character(data$name)[grepl(x,data$name,useBytes=TRUE)]
 		  #    names(res) <- paste("##### Results for ",x,"#####")
 		  if(length(res)>0){return(res)}else{"No study matches your search criteria"}
 	  })
@@ -133,12 +128,17 @@ setMethod(f="getMovebankSensors",
 	  signature=c(study="missing",login="MovebankLogin"), 
 	  definition = function(study,login){
 		  data <- getMovebank("tag_type", login)
-		  #cat("##### LIST OF ALL SENSOR TYPES IN MOVEBANK #####\n")
 		  return(data)
 	  })
 
 setMethod(f="getMovebankSensors", 
-	  signature=c(study="ANY",login="MovebankLogin"), 
+	  signature=c(study="character",login="MovebankLogin"), 
+	  definition = function(study,login){      
+		  study <- getMovebankID(study, login)
+		  callGeneric()
+	  })
+setMethod(f="getMovebankSensors", 
+	  signature=c(study="numeric",login="MovebankLogin"), 
 	  definition = function(study,login){      
 		  data <- getMovebank("sensor", login, tag_study_id=study)
 		  return(data)
@@ -148,24 +148,21 @@ setMethod(f="getMovebankSensors",
 
 setGeneric("getMovebankSensorsAttributes", function(study, login) standardGeneric("getMovebankSensorsAttributes"))
 setMethod(f="getMovebankSensorsAttributes", 
-	  signature=c(study="ANY",login="MovebankLogin"), 
+	  signature=c(study="character",login="MovebankLogin"), 
+	  definition = function(study,login){
+		  study<-getMovebankID(study, login )
+		  callGeneric()
+	  })
+setMethod(f="getMovebankSensorsAttributes", 
+	  signature=c(study="numeric",login="MovebankLogin"), 
 	  definition = function(study,login){
 		  data <- getMovebank("sensor", login, tag_study_id=study)
 		  studySensors <- unique(data$sensor_type_id)
-		  data2 <- lapply(studySensors, function(y, login, study) getMovebank("study_attribute", login, study_id=study, sensor_type_id=y) ,login=login, study=study)
-		  #data2 <- getMovebank("study_attribute", login, study_id=study, sensor_type_id=studySensors[1])
-		  #            if(length(studySensors)>1){
-		  #             for (i in 2:length(studySensors)){ 
-		  #               dataNew <- getMovebank("study_attribute", login, study_id=study, sensor_type_id=studySensors[i])
-		  #               data2<-merge(data2,dataNew)
-		  #             }
-		  #            }         
-		  #cat("##### ATTRIBUTES OF THE SENSORS IN STUDY ID: ",study," \n")
+		  data2 <- lapply(studySensors, function(y, login, study) {try(getMovebank("study_attribute", login, study_id=study, sensor_type_id=y), silent=T)} ,login=login, study=study)
+		  data2<-data2[(lapply(data2, class))!='try-error']
 		  return(as.data.frame(do.call(rbind, data2)))
 	  })
 
-
-
 ###all or a certain ID
 setGeneric("getMovebankID", function(study, login) standardGeneric("getMovebankID"))
 setMethod(f="getMovebankID", 
@@ -180,7 +177,6 @@ setMethod(f="getMovebankID",
 	  definition = function(study=NA, login){
 		  data <- getMovebank("study", login, sort="name", attributes="id%2Cname%2Ci_am_owner%2Ci_can_see_data%2Cthere_are_data_which_i_cannot_see")
 		  if (is.na(study)) {
-			  #cat("####### STUDY ID #######\n")
 			  return(data[ ,c("id","name")])
 		  } else {
 			  studyNUM <- data[gsub(" ","", data$name)==gsub(" ","", study),c("id")] #get rid of all spaces to avoid miss matching between different spaced words
@@ -194,12 +190,17 @@ setMethod(f="getMovebankID",
 ###retrieving information of a certain study
 setGeneric("getMovebankStudy", function(study, login) standardGeneric("getMovebankStudy"))
 setMethod(f="getMovebankStudy", 
-	  signature=c(study="ANY", login="MovebankLogin"),
+	  signature=c(study="numeric", login="MovebankLogin"),
 	  definition = function(study, login){
 		  data <- getMovebank("study", login, id=study)
-		  #cat("**** SUMMARY OF THE REQUESTED STUDY: ",levels(data$name)," ****\n")
 		  return(data)
 	  })
+setMethod(f="getMovebankStudy", 
+	  signature=c(study="character", login="MovebankLogin"),
+	  definition = function(study, login){
+		  study<- getMovebankID(study, login)
+		  callGeneric()
+	  })
 
 setMethod(f="getMovebankStudy", 
 	  signature=c(study="ANY", login="missing"),
@@ -212,9 +213,14 @@ setMethod(f="getMovebankStudy",
 ##get all animals with their IDs
 setGeneric("getMovebankAnimals", function(study, login) standardGeneric("getMovebankAnimals"))
 setMethod(f="getMovebankAnimals",
-	  c(study="ANY", login="MovebankLogin"),
+	  c(study="character", login="MovebankLogin"),
+	  definition = function(study, login){  
+		   study  <- getMovebankID(study,login)
+	  callGeneric()
+	  })
+setMethod(f="getMovebankAnimals",
+	  c(study="numeric", login="MovebankLogin"),
 	  definition = function(study, login){  
-		  if(class(study)=="character") study  <- getMovebankID(study,login)
 		  tags <- getMovebank(entity_type="sensor", login, tag_study_id=study)
 		  animalID <- getMovebank("individual", login, study_id=study, attributes="id%2Clocal_identifier")
 		  deploymentID <- getMovebank("deployment", login=login, study_id=study, attributes="individual_id%2Ctag_id%2Cid")
@@ -241,30 +247,58 @@ setMethod(f="getMovebankAnimals",
 		  getMovebankAnimals(study=study,login=login)
 	  })
 
+setGeneric("getMovebankData", function(study,animalName,login, ...) standardGeneric("getMovebankData"))
 
-
-setGeneric("getMovebankData", function(study,animalName=NA,login, ...) standardGeneric("getMovebankData"))
 setMethod(f="getMovebankData", 
-	  signature=c(study="ANY",animalName="ANY", login="MovebankLogin"),
+	  signature=c(study="ANY",animalName="ANY", login="missing"),
+	  definition = function(study, animalName, login=login, ...){ 
+		  login <- movebankLogin()
+		  callGeneric()
+	  })
+
+setMethod(f="getMovebankData", 
+	  signature=c(study="character",animalName="ANY", login="MovebankLogin"),
 	  definition = function(study, animalName, login, ...){ 
-		  if (class(study)=="character") study <- getMovebankID(study, login) 
-		  idData <- getMovebank("individual", login=login, study_id=study)         
+		  study <- getMovebankID(study, login) 
+		  callGeneric()
+	  })
+
+setMethod(f="getMovebankData", 
+	  signature=c(study="numeric",animalName="missing", login="MovebankLogin"),
+	  definition = function(study, animalName, login, ...){ 
+		  d<- getMovebank("individual", login=login, study_id=study, attributes=c('id'))$id
+		  getMovebankData(study=study, login=login, ..., animalName=d)
+	  })
+setMethod(f="getMovebankData", 
+	  signature=c(study="numeric",animalName="character", login="MovebankLogin"),
+	  definition = function(study, animalName, login, ...){ 
+		  d<- getMovebank("individual", login=login, study_id=study, attributes=c('id','local_identifier'))
+		  animalName<-d[as.character(d$local_identifier)%in%animalName,'id']
+		  callGeneric()
+	  })
+setMethod(f="getMovebankData", 
+	  signature=c(study="numeric",animalName="numeric", login="MovebankLogin"),
+	  definition = function(study, animalName, login, removeDuplicatedTimestamps=F,...){ 
+		  idData <- getMovebank("individual", login=login, study_id=study, id=animalName)         
 		  ##which deployments are imporant
-		  deploymentID <- getMovebank("deployment", login=login, study_id=study, attributes="individual_id%2Ctag_id%2Cid") 
+		  deploymentID <- getMovebank("deployment", login=login, study_id=study, attributes=c("individual_id","tag_id","id"), individual_id=animalName)
 		  ##which track Data are important
-		  sensors <- getMovebankSensors(study=study, login=login)
-		  names(sensors)  <- c("sensor", "sensor_type_id", "tag_id") 
-		  new <- merge.data.frame(deploymentID, sensors, by.x="tag_id", by.y="tag_id") 
-		  new <- merge.data.frame(new, idData, by.x="individual_id", by.y="id")
-		  if (!all(is.na(animalName))) {
-			  new <- new[new$local_identifier%in%animalName, ]
-			  if(length(animalName)!=length(unique(new$individual_id))) stop("One or more animal names are spelled incorrectly.")
+		  new <- merge.data.frame(deploymentID, idData, by.x="individual_id", by.y="id")
+
+		  sensorTypes <- getMovebank("tag_type", login=login)
+		  rownames(sensorTypes)<-sensorTypes$id
+		  locSen <- sensorTypes[as.logical(sensorTypes$is_location_sensor),"id"] #reduce track to location only sensors & only the correct animals
+
+		  attribs <- unique(c(as.character(getMovebankSensorsAttributes(study, login)$short_name),"sensor_type_id","deployment_id",'event_id'))
+
+		  trackDF <- getMovebank("event", login=login, study_id=study, attributes = attribs , deployment_id=new$id, sensor_type_id=locSen)
+		  if(nrow(trackDF)==0){
+			  stop('No records found for this individual/study combination')
 		  }
-		  b <- getMovebank("tag_type", login=login)
-		  locSen <- b[as.logical(b$is_location_sensor),"id"] #reduce track to location only sensors & only the correct animals
-		  attribs <- c(as.character(getMovebankSensorsAttributes(study, login)$short_name),"sensor_type_id","deployment_id")
-		  trackDF <- getMovebank("event", login=login, study_id=study, attributes = attribs , deployment_id=unique(new$id), sensor_type_id=locSen)
 		  trackDF$timestamp<-as.POSIXct(strptime(as.character(trackDF$timestamp), format = "%Y-%m-%d %H:%M:%OS",tz="UTC"), tz="UTC")
+		  if(any(tapply(trackDF$sensor_type_id, trackDF$deployment_id, length)!=1)){# data comes in ordered by sensor but needs to be ordered by timestamp
+			  trackDF <- trackDF[ with(trackDF, order(trackDF$deployment_id, timestamp)) , ]  
+		  }
 		  outliers<- is.na(trackDF$location_long)
 		  if('algorithm_marked_outlier'%in%names(trackDF))
 			  outliers[trackDF$algorithm_marked_outlier=="true"]<-T
@@ -272,6 +306,9 @@ setMethod(f="getMovebankData",
 			  outliers[trackDF$manually_marked_outlier=="true"]<-T
 			  outliers[trackDF$manually_marked_outlier=="false"]<-F
 		  }
+		  if(all(outliers))
+			  stop("There not observed records for this study/individual")
+
 		  spdf<-SpatialPointsDataFrame(trackDF[!outliers,c('location_long','location_lat')], data=trackDF[!outliers,], proj4string=CRS("+proj=longlat +ellps=WGS84 +datum=WGS84"), match.ID=T)
 		  if(any(duplicated(new$local_identifier)))
 			  new$local_identifier<-paste0(new$local_identifier,'_', new$id)
@@ -287,40 +324,34 @@ setMethod(f="getMovebankData",
 			  warning("There are ",sum(s), " duplicated timestamps in the unused that those will be removed")
 			  outliers[outliers][s]<-F
 		  }
-		  name<-b$name
-		  names(name)<-b$id
-		  local_identifier<-factor(rownames(new))
-		  names(local_identifier)<-new$id
+
+		  new<-new[new$id %in% unique(spdf$deployment_id),]
+		  trackId<-droplevels(factor(spdf$deployment_id, labels=rownames(new), levels=new$id))
+		  unUsed<-new('.unUsedRecordsStack', dataUnUsedRecords=trackDF[outliers,],timestampsUnUsedRecords=trackDF$timestamp[outliers], 
+			      sensorUnUsedRecords= sensorTypes[as.character(trackDF[outliers,'sensor_type_id']),'name'],
+			      trackIdUnUsedRecords=factor(trackDF[outliers,'deployment_id'], labels=rownames(new), levels=new$id))
+
+		  if(any(!(s<-(as.character(unUsed@trackIdUnUsedRecords) %in% levels(trackId)))))
+		  {
+			  warning('Omiting individual(s) (n=',length(unique(unUsed@trackIdUnUsedRecords[!s])), ') that have only unUsedRecords')
+			  unUsed<-unUsed[s,]
+		  }
+		  unUsed@trackIdUnUsedRecords<-factor(unUsed@trackIdUnUsedRecords, levels=levels(trackId))
+
+		  if(removeDuplicatedTimestamps){
+			message("removeDupilcatedTimestamps was set to true we strongly suggest against it and that the problem is solved before because there is no systematic to which locations are removed. This can for example be done by marking them outlier in movebank.")
+			dupsDf<-(data.frame(format(spdf$timestamp,"%Y %m %d %H %M %OS4"), spdf$sensor_type_id, trackId))
+			dups<-duplicated(dupsDf)
+			spdf<-spdf[!dups,]
+			trackId<-trackId[!dups]
+			warning(sum(dups)," location(s) is/are removed by removeDuplicatedTimestamps")
+		  }
 
 		  res<-new("MoveStack", spdf, timestamps=spdf$timestamp, 
-			   sensor=name[as.character(spdf$sensor_type_id)],#merge(data.frame(spdf[,'sensor_type_id',drop=F]), b, by.x='sensor_type_id', by.y='id')$name, 
-			   trackId=local_identifier[as.character(spdf$deployment_id)],#merge(data.frame(spdf[,'deployment_id',drop=F]), new, by.x='deployment_id', by.y='id')$local_identifier, 
-			   dataUnUsedRecords=trackDF[outliers,],timestampsUnUsedRecords=trackDF$timestamp[outliers], 
-			   sensorUnUsedRecords= name[as.character(trackDF[outliers,'sensor_type_id'])],#, b, by.x='sensor_type_id', by.y='id')$name, 
-			   trackIdUnUsedRecords=local_identifier[as.character(trackDF[outliers,'deployment_id'])],#, new, by.x='deployment_id', by.y='id')$local_identifier, 
-			   idData=new)
-		  if(length(unique(res@trackId))==1)
+			   sensor=sensorTypes[as.character(spdf$sensor_type_id),'name'],
+			   unUsed,trackId=trackId,
+			   idData=new[as.character(unique(trackId)),])
+		  if(length(n.locs(res))==1)
 			  res<-as(res, 'Move')
 		  return(res)
-		  #       new <- new[new$id%in%unique(trackDF[,"deployment_id"]), ]
-		  #       
-		  #       trackDF <- merge.data.frame(x=trackDF, y=new[, c(names(new)[!names(new)%in%names(trackDF)], "individual_id")], by.x="individual_id", by.y="individual_id", all=TRUE) 
-		  #       trackDF$sensor_type_id <- as.vector(unlist(lapply(trackDF$sensor_type_id, function (y,b){b$external_id[which(b$id==y)]  },b=b))) 
-		  #       if (!any(is.na(new$local_identifier)) & length(unique(new$individual_id))==length(unique(new$local_identifier))) 
-		  #         trackDF$individual_id <- rep(unique(new$local_identifier), unlist(lapply(lapply(unique(trackDF$individual_id), "==", trackDF$individual_id), sum)))            
-		  #       ##multiple sensors per tag
-		  #       #                 if (length(unique(new$id))!=length(unique(new$sensor_id))){
-		  #       #                   trackDF$individual_id  <- paste(trackDF$individual_id, trackDF$deployment_id, trackDF$sensor_type_id, sep="_") ##ADD sensorID!!!
-		  #       #                 } else {
-		  #       ##individuals with multiple deployments?
-		  #       if (length(paste(new$id, new$individual_id, sep="_"))!=length(unique(new$id)))
-		  #         trackDF$individual_id <- paste(trackDF$individual_id, trackDF$deployment_id, trackDF$tag_id, sep="_")
-		  #       studyDF <- getMovebankStudy(study, login)
-		  #       trackDF$study.name <- rep(as.character(studyDF$name),times=nrow(trackDF))
-		  #       trackDF$citation <- rep(as.character(studyDF$citation),times=nrow(trackDF))
-		  #       trackDF$timestamp <- as.POSIXct(strptime(as.character(trackDF$timestamp), format = "%Y-%m-%d %H:%M:%OS",tz="UTC"), tz="UTC")
-		  #       names(trackDF) <- gsub('_', '.', names(trackDF))
-		  #       names(trackDF) <- gsub('local.identifier','individual.local.identifier',names(trackDF))
-		  #       trackDF$study.name <- gsub(' +', " ", trackDF$study.name)
-		  #       .move(df=trackDF, proj=CRS("+proj=longlat +ellps=WGS84 +datum=WGS84"))
 	  })
